# coding=utf-8

import os
import traceback

from pocounit.result.emitter import PocoTestResultEmitter
from pocounit.utils.trace import get_current_lineno_of


class AssertionRecorder(PocoTestResultEmitter):
    TAG = 'assertion'

    def assert_(self, assertionType, args, assertionMsg):
        lineno, srcfilename = get_current_lineno_of(self.collector.get_testcases_filenames())
        self.emit(self.TAG, {'method': 'assert', 'type': assertionType, 'args': args, 'msg': assertionMsg,
                             'lineno': lineno, 'srcfilename': srcfilename})

    def fail(self, errmsg, assertionMsg, tb):
        lineno, srcfilename = get_current_lineno_of(self.collector.get_testcases_filenames())
        self.emit(self.TAG, {'method': 'fail', 'errmsg': errmsg, 'msg': assertionMsg, 'traceback': tb,
                             'lineno': lineno, 'srcfilename': srcfilename})

    def traceback(self, errtype, e, tb):
        # tb1 用来获取最近报错调用帧栈
        # tb用来format一个完整的tb
        # 去到离现场最近的tb位置
        tb1 = tb
        tb_stack = [tb1]
        while tb1.tb_next:
            tb1 = tb1.tb_next
            tb_stack.append(tb1)

        # 跳过unittest相关的tb frame
        while tb and '__unittest' in tb.tb_frame.f_globals:
            tb = tb.tb_next

        lineno, srcfilename = None, None
        if tb:
            while tb_stack:
                lineno, srcfilename = get_current_lineno_of(self.collector.get_testcases_filenames(), tb_stack.pop().tb_frame)
                if srcfilename:
                    break
            if srcfilename:
                srcfilename = os.path.relpath(srcfilename, self.collector.get_project_root_path())
            formatted_tb = ''.join(traceback.format_exception(errtype, e, tb))
        else:
            formatted_tb = ''
        self.emit(self.TAG, {'method': 'traceback', 'errtype': errtype.__name__, 'traceback': formatted_tb,
                             'lineno': lineno, 'srcfilename': srcfilename})



# coding=utf-8

from pocounit.addons import PocoUnitAddon


class ActionTracker(PocoUnitAddon):
    def __init__(self, poco):
        self.poco = poco
        self.action_recorder = None

    def initialize(self, Case):
        self.action_recorder = Case.get_result_emitter('actionRecorder')
        self.poco.add_pre_action_callback(self.on_pre_action)
        self.poco.add_post_action_callback(self.on_post_action)

    def on_pre_action(self, poco, action, proxy, args):
        tid = id(proxy)
        if action in ('click', 'long_click'):
            bounds = proxy.get_bounds()  # t r b l
            self.action_recorder.bounding(tid, bounds)
            self.action_recorder.click(tid, args, repr(proxy))
        elif action == 'swipe':
            bounds = proxy.get_bounds()
            self.action_recorder.bounding(tid, bounds)
            origin, direction = args
            self.action_recorder.swipe(tid, origin, direction, repr(proxy))

    def on_post_action(self, poco, action, proxy, args):
        tid = id(proxy)
        if action in ('click', 'long_click', 'swipe'):
            self.action_recorder.clear(tid)



# coding=utf-8

import sys
from unittest.runner import TextTestResult


class PocoTestResult(TextTestResult):
    def __init__(self, stream=sys.stderr, descriptions=True, verbosity=1):
        super(PocoTestResult, self).__init__(stream, descriptions, verbosity)
        self.detail_errors = []

    def addError(self, test, err):
        exc_type, e, exc_tb = err
        self.detail_errors.append((test, exc_type, e, exc_tb))
        return super(PocoTestResult, self).addError(test, err)



# coding=utf-8
from __future__ import print_function

import os
import inspect

from pocounit.case import PocoTestCase
from pocounit.runner import PocoTestRunner
from pocounit.suite import PocoTestSuite

from pocounit.utils.misc import has_override


def run(suite):
    if isinstance(suite, PocoTestCase):
        case = suite
        suite = PocoTestSuite()
        suite.addTest(case)
    runner = PocoTestRunner()
    return runner.run(suite)


def main():
    # testcase detection
    current_frame = inspect.currentframe()
    caller = current_frame.f_back
    test_case_filename = os.path.abspath(caller.f_code.co_filename)  # 脚本务必是绝对路径才行
    caller_scope = caller.f_locals
    print('this testcase filename is "{}".'.format(test_case_filename))

    # 这部分代码放到loader里
    Cases = [v for k, v in caller_scope.items()
             if type(v) is type
             and v is not PocoTestCase
             and issubclass(v, PocoTestCase)
             and has_override("runTest", v, PocoTestCase)
             ]

    suite = PocoTestSuite()
    for Case in Cases:
        case = Case()
        suite.addTest(case)

    runner = PocoTestRunner()
    success = runner.run(suite)

    if not success:
        exit(-1)




# coding=utf-8

import sys

from pocounit.result.emitter import PocoTestResultEmitter


class Redirector(object):
    def __init__(self, origin, _type, logger):
        super(Redirector, self).__init__()
        self.buf = ''
        self.type = _type
        self.logger = logger
        if hasattr(sys, '__save_origin_std_' + _type):
            self.origin = getattr(sys, '__save_origin_std_' + _type)
        else:
            self.origin = origin
            setattr(sys, '__save_origin_std_' + _type, origin)

    def write(self, text):
        # 如果原流不支持unicode，那就忽略
        try:
            self.origin.write(text)
        except:
            pass

        self.buf += text
        lines = None
        if self.buf.endswith('\n'):
            lines, self.buf = self.buf.rstrip(), ''
        elif '\n' in self.buf:
            lines, self.buf = self.buf.rsplit('\n', 1)

        if lines:
            self.logger.log(lines)

    def __getattr__(self, key):
        if hasattr(self.origin, key):
            return getattr(self.origin, key)
        else:
            raise AttributeError('{} has no attribute {}'.format(self.origin, key))


class RunnerRuntimeLog(PocoTestResultEmitter):
    TAG = 'runnerRuntimeLog'

    def __init__(self, collector):
        super(RunnerRuntimeLog, self).__init__(collector)

    def start(self):
        if not hasattr(sys, '__redirected_stdout'):
            setattr(sys, '__redirected_stdout', sys.stdout)
            sys.stdout = Redirector(sys.stdout, 'stdout', self)
        if not hasattr(sys, '__redirected_stderr'):
            setattr(sys, '__redirected_stderr', sys.stderr)
            sys.stderr = Redirector(sys.stderr, 'stderr', self)

    def stop(self):
        if hasattr(sys, '__redirected_stdout'):
            sys.stdout = sys.stdout.origin
            delattr(sys, '__redirected_stdout')
        if hasattr(sys, '__redirected_stderr'):
            sys.stderr = sys.stderr.origin
            delattr(sys, '__redirected_stderr')

    def log(self, content):
        self.emit(self.TAG, {'content': content})



# coding=utf-8

import sys
import traceback
import unittest
import warnings

from unittest.util import safe_repr

from pocounit.fixture import FixtureUnit
from pocounit.result import PocoTestResult
from pocounit.result.metainfo import MetaInfo
from pocounit.result.collector import PocoResultCollector
from pocounit.result.trace import ScriptTracer
from pocounit.result.record import ScreenRecorder
from pocounit.result.action import ActionRecorder
from pocounit.result.app_runtime import AppRuntimeLog
from pocounit.result.runner_runtime import RunnerRuntimeLog
from pocounit.result.assertion import AssertionRecorder


class PocoTestCase(unittest.TestCase, FixtureUnit):
    """
    * longMessage: determines whether long messages (including repr of
        objects used in assert methods) will be printed on failure in *addition*
        to any explicit message passed.
    """
    longMessage = True

    _resule_collector = None
    _result_emitters = None     # {name -> PocoTestResultEmitter}
    _addons = None              # [addons]

    def __init__(self):
        unittest.TestCase.__init__(self)
        FixtureUnit.__init__(self)

        collector = PocoResultCollector(self.project_root, [self.test_case_filename], self.name(), self.test_case_dir)
        self.set_result_collector(collector)

        meta_info_emitter = MetaInfo(collector)
        runner_runtime_log = RunnerRuntimeLog(collector)
        tracer = ScriptTracer(collector)
        screen_recorder = ScreenRecorder(collector)
        action_recorder = ActionRecorder(collector)
        app_runtime_log = AppRuntimeLog(collector)
        assertion_recorder = AssertionRecorder(collector)

        self.add_result_emitter('metaInfo', meta_info_emitter)
        self.add_result_emitter('runnerRuntimeLog', runner_runtime_log)
        self.add_result_emitter('tracer', tracer)
        self.add_result_emitter('screenRecorder', screen_recorder)
        self.add_result_emitter('actionRecorder', action_recorder)
        self.add_result_emitter('appRuntimeLog', app_runtime_log)
        self.add_result_emitter('assertionRecorder', assertion_recorder)

        self.meta_info_emitter = meta_info_emitter

    @classmethod
    def name(cls):
        """
        改写此方法来自定义testcase的名字，允许中文，请使用utf-8编码的str或者unicode

        :return:
        """

        if type(cls) is type:
            return cls.__name__
        else:
            return cls.__class__.__name__

    @classmethod
    def setUpClass(cls):
        """
        用例预处理，加载自定义插件等

        :return: 
        """
        pass

    def runTest(self):
        """
        testcase的正文，把要执行的测试和包括断言语句都写到这里

        """

        raise NotImplementedError

    @classmethod
    def tearDownClass(cls):
        """
        用例后处理

        :return: 
        """
        pass

    def shortDescription(self):
        """
        描述这个testcase的细节，如果需要的话就override这个方法
        
        :return: <str>
        """

        return super(PocoTestCase, self).shortDescription()

    def defaultTestResult(self):
        return PocoTestResult()

    @classmethod
    def set_result_collector(cls, collector):
        if type(cls) is not type:
            cls = cls.__class__
        cls._resule_collector = collector

    @classmethod
    def get_result_collector(cls):
        return cls._resule_collector

    @classmethod
    def add_result_emitter(cls, name, emitter):
        if type(cls) is not type:
            cls = cls.__class__
        if not cls._result_emitters:
            cls._result_emitters = {}
        cls._result_emitters[name] = emitter

    @classmethod
    def get_result_emitter(cls, name):
        return cls._result_emitters.get(name)

    @classmethod
    def register_addon(cls, addon):
        if type(cls) is not type:
            cls = cls.__class__
        if not cls._addons:
            cls._addons = []
        cls._addons.append(addon)

    @classmethod
    def register_addin(cls, v):
        warnings.warn('`register_addin` is deprecated. Please use `register_addon` instead.')
        return cls.register_addon(v)

    def run(self, result=None):
        result = result or self.defaultTestResult()
        if not isinstance(result, PocoTestResult):
            raise TypeError('Test result class should be subclass of PocoTestResult. '
                            'Current test result instance is "{}".'.format(result))

        # 自动把当前脚本add到collector的脚本watch列表里
        collector = self.get_result_collector()
        collector.add_testcase_file(self.test_case_filename)

        # register addon
        for addon in self._addons:
            addon.initialize(self)

        self.meta_info_emitter.test_started(self.__class__.__name__)

        # start result emitter
        for name, emitter in self._result_emitters.items():
            try:
                emitter.start()
            except Exception as e:
                warnings.warn('Fail to start result emitter: "{}". Error message: \n"{}"'
                              .format(emitter.__class__.__name__, e.message))

        # run test
        ex = None
        ret = None
        try:
            ret = super(PocoTestCase, self).run(result)
        except Exception as e:
            ex = e

        assertionRecorder = self.get_result_emitter('assertionRecorder')
        for _, exc_type, e, tb in result.detail_errors:
            assertionRecorder.traceback(exc_type, e, tb)

        # stop result emitter
        for name, emitter in self._result_emitters.items():
            try:
                emitter.stop()
            except:
                pass

        self.meta_info_emitter.test_ended(self.__class__.__name__)

        # handle result
        if ex is not None:
            self.meta_info_emitter.test_fail(self.__class__.__name__)
            raise ex
        else:
            self.meta_info_emitter.test_succeed(self.__class__.__name__)
            return ret

    def fail(self, msg=None):
        assertionRecorder = self.get_result_emitter('assertionRecorder')
        try:
            return super(PocoTestCase, self).fail(msg)
        except:
            exc_type, e, exc_tb = sys.exc_info()
            assertionRecorder.fail(msg, str(e), traceback.format_exc())
            raise exc_type, msg, exc_tb

    # assertions
    def assertTrue(self, expr, msg=None):
        assertionRecorder = self.get_result_emitter('assertionRecorder')
        assertionRecorder.assert_('True', [expr], msg)

        if not expr:
            msg = self._formatMessage(msg, "%s is not true" % safe_repr(expr))
            self.fail(msg)

    def assertFalse(self, expr, msg=None):
        assertionRecorder = self.get_result_emitter('assertionRecorder')
        assertionRecorder.assert_('False', [expr], msg)

        if expr:
            msg = self._formatMessage(msg, "%s is not false" % safe_repr(expr))
            self.fail(msg)

    def assertEqual(self, first, second, msg=None):
        assertionRecorder = self.get_result_emitter('assertionRecorder')
        assertionRecorder.assert_('Equal', [first, second], msg)
        return super(PocoTestCase, self).assertEqual(first, second, msg)

    def assertNotEqual(self, first, second, msg=None):
        assertionRecorder = self.get_result_emitter('assertionRecorder')
        assertionRecorder.assert_('NotEqual', [first, second], msg)
        return super(PocoTestCase, self).assertNotEqual(first, second, msg)

    def assertLess(self, a, b, msg=None):
        assertionRecorder = self.get_result_emitter('assertionRecorder')
        assertionRecorder.assert_('Less', [a, b], msg)
        return super(PocoTestCase, self).assertLess(a, b, msg)

    def assertLessEqual(self, a, b, msg=None):
        assertionRecorder = self.get_result_emitter('assertionRecorder')
        assertionRecorder.assert_('LessEqual', [a, b], msg)
        return super(PocoTestCase, self).assertLessEqual(a, b, msg)

    def assertGreater(self, a, b, msg=None):
        assertionRecorder = self.get_result_emitter('assertionRecorder')
        assertionRecorder.assert_('Greater', [a, b], msg)
        return super(PocoTestCase, self).assertGreater(a, b, msg)

    def assertGreaterEqual(self, a, b, msg=None):
        assertionRecorder = self.get_result_emitter('assertionRecorder')
        assertionRecorder.assert_('GreaterEqual', [a, b], msg)
        return super(PocoTestCase, self).assertGreaterEqual(a, b, msg)

    def assertIn(self, member, container, msg=None):
        assertionRecorder = self.get_result_emitter('assertionRecorder')
        assertionRecorder.assert_('In', [member, container], msg)
        return super(PocoTestCase, self).assertIn(member, container, msg)

    def assertNotIn(self, member, container, msg=None):
        assertionRecorder = self.get_result_emitter('assertionRecorder')
        assertionRecorder.assert_('NotIn', [member, container], msg)
        return super(PocoTestCase, self).assertNotIn(member, container, msg)



# coding=utf-8

import os

from pocounit.result.emitter import PocoTestResultEmitter
from airtest.core.api import device as current_device


class ScreenRecorder(PocoTestResultEmitter):
    TAG = 'record'

    def __init__(self, collector, device=None):
        super(ScreenRecorder, self).__init__(collector)
        self.device = device or current_device()
        self.record_filepath = os.path.join(self.collector.get_root_path(), 'screen.mp4')
        self.started = False

    def start(self):
        if self.started:
            return False
        self.started = True

        self.device = self.device or current_device()
        if self.device and hasattr(self.device, 'start_recording'):
            try:
                # force to stop before starting new recoding
                self.device.stop_recording(self.record_filepath)
            except:
                pass

            success = self.device.start_recording()
            if success:
                relpath = self.collector.get_resource_relative_path(self.record_filepath)
                display_info = self.device.display_info
                if display_info['orientation'] in (1, 3):
                    orientation = 'horizontal'
                    resolution = [display_info['height'], display_info['width']]
                else:
                    orientation = 'vertical'
                    resolution = [display_info['width'], display_info['height']]

                # orientation: current orientation
                # resolution: recording resolution. 当前默认等于屏幕分辨率
                relpath = relpath.replace('\\', '/')
                self.emit(self.TAG, {'event': 'started', 'filename': relpath, 'orientation': orientation, 'resolution': resolution})
            return success
        return False

    def stop(self):
        if not self.started:
            return False
        self.started = False

        self.device = self.device or current_device()
        if self.device and hasattr(self.device, 'stop_recording'):
            self.emit(self.TAG, {'event': 'stopped'})
            try:
                return self.device.stop_recording(self.record_filepath)
            except:
                try:
                    # 如果停止失败，就尝试直接pull最后一次录屏文件
                    return self.device.recorder.pull_last_recording_file(self.record_filepath)
                except:
                    pass
        return False



# coding=utf-8

"""
这个文件别引用，未完成
"""

import inspect
import sys
import traceback

from pocounit.addons import PocoUnitAddon
from pocounit.addons.airtest.utils import Hooker

from airtest.core.api import loop_find, touch, swipe, exists, assert_exists
from airtest.core.cv import _cv_match
from airtest.core.api import Template as Target
from airtest.cli.runner import device as current_device


class ActionTracker(PocoUnitAddon):
    """
    general airtest action pattern and invocation stack

    |  touch start
    |    loop find
    |      cv match, return match result
    |  touch end

    """

    def __init__(self):
        self.action_recorder = None
        self.assertion_recorder = None
        self._doing_actions = False  # 在touch或swipe中时设为true

        self.loop_find_ = Hooker(loop_find)
        self.touch_ = Hooker(touch)
        self._cv_match_ = Hooker(_cv_match)
        self.assert_exists_ = Hooker(assert_exists)

        self.touch_.pre(self.pre_touch)
        self.touch_.post(self.post_touch)
        self.loop_find_.post(self.post_loop_find)
        self._cv_match_.post(self.post_cv_match)
        self.assert_exists_.pre(self.pre_assert_exists)
        self.assert_exists_.post(self.post_assert_exists)

        import airtest.core.main as acm
        import airtest.core.cv as acc
        acm.touch = self.touch_
        acm.loop_find = self.loop_find_
        acm.assert_exists = self.assert_exists_
        acc._cv_match = self._cv_match_

    def initialize(self, Case):
        self.action_recorder = Case.get_result_emitter('actionRecorder')
        self.assertion_recorder = Case.get_result_emitter('assertionRecorder')

        frame = sys._getframe(0)
        while frame.f_back is not None:
            frame.f_globals['touch'] = self.touch_
            frame.f_globals['loop_find'] = self.loop_find_
            frame.f_globals['assert_exists'] = self.assert_exists_
            frame = frame.f_back

    def post_loop_find(self, pos, v, *args, **kwargs):
        if pos is not None and isinstance(v, Target):
            w, h = current_device().getCurrentScreenResolution()
            x, y = pos
            tid = id(v)
            if self._doing_actions:
                self.action_recorder.click(tid, [1.0 * x / w, 1.0 * y / h], v.filepath)

    def post_cv_match(self, cv_ret, *args, **kwargs):
        if not cv_ret:
            return

        # 如果当前函数是由loop_find调用的，那就可以找到一个rect，这个rect由airtest.core.cv._cv_match里给出
        # 以下就是从frame stack中一直找到loop_find这一帧，然后找出loop_find的第一个argument，通过argument求出tid
        frame = sys._getframe(0)
        while frame and frame.f_code.co_name != 'loop_find':
            frame = frame.f_back
        if frame:
            # more details about inspect parameter name in runtime,
            # see https://docs.python.org/2/library/inspect.html#inspect.getargvalues
            args, varargs, keywords, locals = inspect.getargvalues(frame)
            if len(args) > 0:
                v_name = args[0]
            elif varargs is not None and len(locals[varargs]) > 0:
                v_name = locals[varargs][0]
            else:
                raise ValueError('loop_find第一个参数不支持使用keyword args')

            # 取到loop_find的第一个argument
            v = locals[v_name]
            tid = id(v)
            rect = cv_ret.get("rectangle")
            if rect:
                # a rect's each vertex in screen as following
                # [0]  [3]
                # [1]  [2]
                t = rect[0][1] * 1.0
                r = rect[3][0] * 1.0
                b = rect[1][1] * 1.0
                l = rect[0][0] * 1.0
                w, h = current_device().getCurrentScreenResolution()
                self.action_recorder.bounding(tid, [t / h, r / w, b / h, l / w])

    def pre_touch(self, v, *args, **kwargs):
        self._doing_actions = True
        if isinstance(v, (list, tuple)):
            tid = id(v)
            w, h = current_device().getCurrentScreenResolution()
            x, y = v
            self.action_recorder.click(tid, [1.0 * x / w, 1.0 * y / h], 'fixed point: {}'.format(v))

    def post_touch(self, ret, v, *args, **kwargs):
        self._doing_actions = False
        if isinstance(v, (Target, list, tuple)):
            tid = id(v)
            self.action_recorder.clear(tid)
            return

    def pre_assert_exists(self, v, message, *args, **kwargs):
        if isinstance(v, Target):
            pass

    def post_assert_exists(self, ret, v, message, *args, **kwargs):
        if isinstance(v, Target):
            tid = id(v)
            self.assertion_recorder.assert_('Existence', [v.filename], message)
            self.action_recorder.clear(tid)



# coding=utf-8


class PocoTestResultEmitter(object):
    def __init__(self, collector):
        self.collector = collector

    def start(self):
        pass

    def stop(self):
        pass

    def finalize(self):
        pass

    def emit(self, tag, value):
        self.collector.collect(tag, value)



# coding=utf-8

from pocounit.result.emitter import PocoTestResultEmitter


class AppRuntimeLog(PocoTestResultEmitter):
    TAG = 'appRuntimeLog'

    def __init__(self, collector):
        super(AppRuntimeLog, self).__init__(collector)
        self.started = False

    def start(self):
        self.started = True

    def stop(self):
        self.started = False

    def log(self, content):
        if self.started:
            self.emit(self.TAG, {'content': content})



# coding=utf-8

import os
import inspect

from pocounit.assets_manager import AssetsManager
from pocounit.utils.misc import get_project_root


class FixtureUnit(object):
    _assets_manager = None

    def __init__(self):
        test_case_filename = os.path.abspath(inspect.getsourcefile(self.__class__))
        test_case_dir = os.path.dirname(test_case_filename)
        project_root = get_project_root(test_case_filename)
        print('using "{}" as project root. This testcase is "{}"'.format(project_root, self.__class__.__name__))
        self.set_assets_manager(AssetsManager(project_root))

        self.test_case_filename = test_case_filename
        self.test_case_dir = test_case_dir
        self.project_root = project_root

    def setUp(self):
        """
        初始化一个testcase
        不要把测试逻辑放到这里写，setUp报错的话也相当于整个case报错

        """

        pass

    def tearDown(self):
        """
        一个testcase的清场工作

        """

        pass

    @classmethod
    def set_assets_manager(cls, v):
        if type(cls) is not type:
            cls = cls.__class__
        cls._assets_manager = v

    @classmethod
    def get_assets_manager(cls):
        return cls._assets_manager

    @classmethod
    def R(cls, respath):
        return cls._assets_manager.get_resource_path(respath)



# coding=utf-8
from __future__ import unicode_literals

from pocounit.addons import PocoUnitAddon
from hunter_cli.device_output import trace


class AppRuntimeLogging(PocoUnitAddon):
    def __init__(self, hunter):
        self.hunter = hunter
        self.logger = None

    def initialize(self, Case):
        self.logger = Case.get_result_emitter('appRuntimeLog')
        central_server_url = 'http://hunter.nie.netease.com/webterm'
        trace(self.hunter.tokenid, self.hunter.device_info['id'], central_server_url, self.on_app_runtime_log)

    def on_app_runtime_log(self, data):
        content = '[{data[level]}] {data[data]}\n'.format(data=data)
        self.logger.log(content)



# coding=utf-8

import os
import time
import json

from pocounit.result.logger import StreamLogger


class PocoResultCollector(object):
    def __init__(self, project_root, testcases_filenames, testcase_name, testcase_dir='.',
                 logfilename='poco-result.log',
                 metainfofilename='metainfo.txt'):
        self.testcases_filenames = set(testcases_filenames)  # [full path], multiple files
        self.testcase_dir = testcase_dir
        self.project_root = project_root

        root_paths = [project_root, 'pocounit-results']
        root_paths += os.path.relpath(testcase_dir, project_root).replace('\\', '/').split('/')
        root_paths.append(os.path.splitext(os.path.basename(testcases_filenames[0]))[0])
        root_paths.append(testcase_name)

        self.root = os.path.join(*root_paths)
        if os.path.isfile(self.root):
            raise RuntimeError('"{}" already exists as non-directory, please remove it first.')
        if not os.path.exists(self.root):
            os.makedirs(self.root)
        self.logger = StreamLogger(os.path.join(self.root, logfilename))
        self.metainfo_logger = StreamLogger(os.path.join(self.root, metainfofilename))

    def collect(self, tag, value):
        self.logger.write(json.dumps({'tag': tag, 'value': value, 'ts': time.time()}))

    def collect_metainfo(self, tag, value):
        self.metainfo_logger.write(json.dumps({'tag': tag, 'value': value}))

    def get_root_path(self):
        return self.root

    def get_project_root_path(self):
        return self.project_root

    def get_resource_relative_path(self, respath):
        return os.path.relpath(respath, self.root)

    def get_testcases_filenames(self):
        return self.testcases_filenames

    def add_testcase_file(self, path):
        """
        除了TestCase class所定义的那个文件之外，的其他关联的文件都可以加进来

        :param path:
        :return:
        """

        if not os.path.isabs(path):
            path = os.path.join(self.project_root, path)
        self.testcases_filenames.add(path)



# coding=utf-8

import os
import sys
import shutil

from pocounit.result.emitter import PocoTestResultEmitter


class ScriptTracer(PocoTestResultEmitter):
    TAG = 'trace'

    def __init__(self, collector):
        super(ScriptTracer, self).__init__(collector)
        self.project_root = self.collector.get_project_root_path()
        self._script_filenames = None
        self._script_filenames_lower = None
        self._origin_trace_func = None

    def start(self):
        self._script_filenames = self.collector.get_testcases_filenames()
        self._script_filenames_lower = [f.lower().replace('\\', '/') for f in self._script_filenames]
        for f in self._script_filenames:
            src = os.path.relpath(f, self.project_root)
            dst = os.path.join(self.collector.get_root_path(), src)
            dst_dir = os.path.dirname(dst)
            if not os.path.exists(dst_dir):
                os.makedirs(dst_dir)
            shutil.copyfile(f, dst)
        self._origin_trace_func = sys.gettrace()
        sys.settrace(self._make_tracer())

    def stop(self):
        sys.settrace(self._origin_trace_func)

    def _make_tracer(self):
        def tracer(frame, event, arg):
            co_filename = frame.f_code.co_filename
            if event == 'line' and co_filename.lower().replace('\\', '/') in self._script_filenames_lower:
                line_num = frame.f_lineno
                fname = os.path.relpath(co_filename, self.project_root)
                fname = fname.replace('\\', '/')
                self.emit(self.TAG, {'filename': fname, 'lineno': line_num})
            return tracer
        return tracer



# coding=utf-8

import unittest

from pocounit.result import PocoTestResult


class PocoTestRunner(unittest.TextTestRunner):
    """
    先占坑，先默认用text方式展示，以后再换成自定义的    
    """

    resultclass = PocoTestResult



# coding=utf-8

from pocounit.result.emitter import PocoTestResultEmitter
from pocounit.utils.trace import get_current_lineno_of


class ActionRecorder(PocoTestResultEmitter):
    TAG = 'action'

    def motion(self, tid, desc, points):
        """
        记录motion event，一个motion event就是一个点的轨迹，比如从A拖动到B，每个motion event使用tid分组，同一个tid分组的会归并在一起，
        多点触控的两组轨迹就必须是相同的tid，调用两次这个方法并传入相同的tid即可

        :param tid: event 分组标号
        :param desc: 事件描述
        :param points: 点序列，按顺序构成一系列轨迹
        :return: None
        """

        lineno, srcfilename = get_current_lineno_of(self.collector.get_testcases_filenames())
        self.emit(self.TAG, {'action': 'click', 'tid': tid, 'targetDescription': desc, 'points': points,
                             'lineno': lineno, 'srcfilename': srcfilename})

    def key(self, tid, keyname):
        """
        记录一个按键事件

        :param tid: 事件编号，配合clear清除 
        :param keyname:  按键键名，如 A B Enter等
        :return: None
        """

        raise NotImplementedError

    def click(self, tid, pos, desc):
        lineno, srcfilename = get_current_lineno_of(self.collector.get_testcases_filenames())
        self.emit(self.TAG, {'action': 'click', 'tid': tid, 'origin': pos, 'targetDescription': desc,
                             'lineno': lineno, 'srcfilename': srcfilename})

    def swipe(self, tid, origin, direction, desc):
        lineno, srcfilename = get_current_lineno_of(self.collector.get_testcases_filenames())
        self.emit(self.TAG, {'action': 'swipe', 'tid': tid, 'origin': origin, 'direction': direction, 'targetDescription': desc,
                             'lineno': lineno, 'srcfilename': srcfilename})

    def bounding(self, tid, bound):
        self.emit(self.TAG, {'action': 'bounding', 'tid': tid, 'bounds': bound})

    def clear(self, tid):
        self.emit(self.TAG, {'action': 'clear', 'tid': tid})



# coding=utf-8

import unittest

from pocounit.fixture import FixtureUnit


class PocoTestSuite(unittest.TestSuite, FixtureUnit):
    def __init__(self, tests=()):
        unittest.TestSuite.__init__(self, tests)
        FixtureUnit.__init__(self)

    def run(self, result, debug=False):
        self.setUp()
        super(PocoTestSuite, self).run(result, debug)
        self.tearDown()



# coding=utf-8

import os
import re


def detect_package_root(mod_filename):
    """
    从某个模块的文件路径向上搜索，一直找到这个模块所在包的根目录，将包名第一段返回

    :param mod_filename:
    :return: 根包名 or None
    """

    egginfo_pattern = re.compile(r'.*\.egg-info$')
    p = os.path.abspath(os.path.join(mod_filename, '..'))
    while True:
        ls = os.listdir(p)
        matched = False
        for f in ls:
            if f == 'setup.py' or egginfo_pattern.match(f):
                matched = True
                break
        if matched:
            break
        p_prev = os.path.abspath(os.path.join(p, '..'))
        if p == p_prev:
            return None
        p = p_prev
    return p


def get_project_root(test_case_filename):
    return os.getenv("PROJECT_ROOT") or detect_package_root(test_case_filename) or os.getcwd()


def has_override(method, subCls, baseCls):
    return getattr(subCls, method).__func__ is not getattr(baseCls, method).__func__



# coding=utf-8


import unittest

from pocounit.case import PocoTestCase


class TestPocoRunner(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        pass

    def test_run(_):
        class MyCase(PocoTestCase):
            def setUp(self):
                print('setup')
                self.value = '2333'

            def runTest(self):
                print('before')
                self.assertEqual(1, 1, 'ok')
                self.assertEqual(self.value, '2333', 'setup ok')
                print('after')

            def tearDown(self):
                print('teardown')

        suite = unittest.TestSuite()
        suite.addTest(MyCase())

        runner = unittest.TextTestRunner()
        runner.run(suite)



# coding=utf-8

import threading


class StreamLogger(object):
    def __init__(self, filename='poco-stream-result.log'):
        self.logfile = open(filename, 'w')
        self.mutex = threading.Lock()

    def write(self, val):
        with self.mutex:
            self.logfile.write(val)
            self.logfile.write('\n')
            self.logfile.flush()



# coding=utf-8

import datetime

from pocounit.result.emitter import PocoTestResultEmitter


class MetaInfo(PocoTestResultEmitter):
    TAG = 'MetaInfo'

    def __init__(self, collector):
        super(MetaInfo, self).__init__(collector)

    def test_started(self, name):
        self.emit(self.TAG, {
            'type': 'testcase',
            'value': {
                'name': name,
                'started_at': str(datetime.datetime.now()),
            }
        })

    def test_ended(self, name):
        self.emit(self.TAG, {
            'type': 'testcase',
            'value': {
                'name': name,
                'ended_at': str(datetime.datetime.now()),
            }
        })

    def test_succeed(self, name):
        self.emit(self.TAG, {
            'type': 'testcase',
            'value': {
                'name': name,
                'success': True,
            }
        })

    def test_fail(self, name):
        self.emit(self.TAG, {
            'type': 'testcase',
            'value': {
                'name': name,
                'success': False,
            }
        })

    def snapshot_device_info(self, udid, info):
        """
        Take down the device info according udid.

        :param udid: serialno for Android
        :param info: a dict hold any information of the device
        """

        self.emit(self.TAG, {
            'type': 'device',
            'value': {
                'udid': udid,
                'info': info,
            }
        })

    def emit(self, tag, value):
        self.collector.collect_metainfo(tag, value)



# coding=utf-8

import os


class AssetsManager(object):
    def __init__(self, project_root):
        super(AssetsManager, self).__init__()
        self.project_root = project_root

    def get_abspath(self, p):
        return os.path.join(self.project_root, p)

    get_resource_path = get_abspath



# coding=utf-8

from pocounit.case import PocoTestCase
from pocounit.addons.poco.action_tracking import ActionTracker

from poco.drivers.android.uiautomation import AndroidUiautomationPoco


class AndroidNativeUITestCase(PocoTestCase):
    @classmethod
    def setUpClass(cls):
        super(AndroidNativeUITestCase, cls).setUpClass()

        cls.poco = AndroidUiautomationPoco()

        # 启用动作捕捉(action tracker)和游戏运行时日志捕捉插件(runtime logger)
        action_tracker = ActionTracker(cls.poco)
        cls.register_addon(action_tracker)


class T1(AndroidNativeUITestCase):
    def runTest(self):
        self.poco(text='设置').click()
        self.poco(text='WLAN').click()


if __name__ == '__main__':
    import pocounit
    pocounit.main()



# coding=utf-8

import sys


def get_current_lineno_of(filenames, frame=None):
    filenames_lower = [f.lower().replace('\\', '/') for f in filenames]

    frame = frame or sys._getframe(0)
    while frame:
        if frame.f_code.co_filename.lower().replace('\\', '/') in filenames_lower:
            break
        frame = frame.f_back

    if frame:
        return frame.f_lineno, frame.f_code.co_filename.replace('\\', '/')
    else:
        return None, None



# coding=utf-8


def preload_airtest_device_context(run_on_win=False, window_title='^.*errors and.*$'):
    from airtest.cli.runner import device as current_device
    if not current_device():
        from airtest.core.main import set_serialno, set_windows
        from airtest.core.settings import Settings
        from airtest.core.android.adb import ADB
        adb_client = ADB()
        available_devices = adb_client.devices('device')
        if run_on_win or not available_devices:
            Settings.FIND_INSIDE = [8, 30]  # 窗口边框偏移
            set_windows(window_title=window_title)
        else:
            set_serialno()
        exec("from airtest.core.main import *") in globals()





# coding=utf-8

from setuptools import setup, find_packages


def parse_requirements(filename='requirements.txt'):
    """ load requirements from a pip requirements file. (replacing from pip.req import parse_requirements)"""
    lineiter = (line.strip() for line in open(filename))
    return [line for line in lineiter if line and not line.startswith("#")]


setup(
    name='pocounit',
    version='1.0.21',
    keywords="PocoUnit unittest",
    description='Unittest framework for poco and airtest',
    packages=find_packages(),
    include_package_data=True,
    install_requires=parse_requirements(),
    license='Apache License 2.0',
)



# coding=utf-8

import sys


class Hooker(object):
    def __init__(self, func):
        self.pre_func = None
        self.hooked_func = func
        self.post_func = None
        self.exc_info = None

    def __call__(self, *args, **kwargs):
        if self.pre_func:
            self.pre_func(*args, **kwargs)

        ret = None
        try:
            ret = self.hooked_func(*args, **kwargs)
        except:
            self.exc_info = sys.exc_info()

        if self.post_func:
            self.post_func(ret, *args, **kwargs)

        if self.exc_info:
            raise self.exc_info
        else:
            return ret

    def pre(self, func):
        self.pre_func = func
        return func

    def post(self, func):
        self.post_func = func
        return func

    def __repr__(self):
        return '<{}.{} for {} object @+id/0x{}>'.format(self.__module__, self.__class__.__name__,
                                                        repr(self.hooked_func), hex(id(self)))
    __str__ = __repr__
